@@ -13,10 +13,15 @@ class TemplatesApi(CseClient):
         super().__init__(client)
         self._uri = f"{self._uri}/{shared_constants.CSE_URL_FRAGMENT}"
         self._templates_uri = f"{self._uri}/templates"
+        self._tkgm_templates_uri = f"{self._templates_uri}/tkgm"
 
-    def list_templates(self):
+    def list_templates(self, is_tkgm: bool):
+        if is_tkgm:
+            uri = self._tkgm_templates_uri
+        else:
+            uri = self._templates_uri
         response = self.do_request(
-            uri=self._templates_uri,
+            uri=uri,
             method=shared_constants.RequestMethod.GET,
             accept_type='application/json')
         return self.process_response(response)
