@@ -16,6 +16,8 @@ import container_service_extension.client.tkgclient.rest as tkg_rest
 import container_service_extension.client.utils as client_utils
 from container_service_extension.common.constants.shared_constants import CSE_PAGINATION_DEFAULT_PAGE_SIZE, PaginationKey  # noqa: E501
 from container_service_extension.common.constants.shared_constants import CSE_PAGINATION_FIRST_PAGE_NUMBER  # noqa: E501
+from container_service_extension.common.constants.shared_constants import EntityTypeId  # noqa: E501
+import container_service_extension.common.utils.core_utils as core_utils
 import container_service_extension.common.utils.pyvcloud_utils as vcd_utils
 import container_service_extension.exception.exceptions as cse_exceptions
 import container_service_extension.logging.logger as logger
@@ -41,14 +43,18 @@ class DECluster:
     """
 
     def __init__(self, client):
-        logger_wire = logger.NULL_LOGGER
-        if os.getenv(cli_constants.ENV_CSE_CLIENT_WIRE_LOGGING):
-            logger_wire = logger.CLIENT_WIRE_LOGGER
+        logger_wire = logger.CLIENT_WIRE_LOGGER \
+            if core_utils.str_to_bool(
+                os.getenv(cli_constants.ENV_CSE_CLIENT_WIRE_LOGGING)
+            ) \
+            else logger.NULL_LOGGER
         self._client = client
         self._cloudapi_client = \
             vcd_utils.get_cloudapi_client_from_vcd_client(
-                client=client, logger_debug=logger.CLIENT_LOGGER,
-                logger_wire=logger_wire)
+                client=client,
+                logger_debug=logger.CLIENT_LOGGER,
+                logger_wire=logger_wire
+            )
         self._nativeCluster = DEClusterNative(client)
         self._tkgCluster = DEClusterTKGS(client)
         schema_svc = def_schema_svc.DefSchemaService(self._cloudapi_client)
@@ -95,7 +101,9 @@ class DECluster:
                         version=common_models.K8Interface.VCD_INTERFACE.value.version,  # noqa: E501
                         filters=filters,
                         page_number=page_number,
-                        page_size=page_size)
+                        page_size=page_size,
+                        exclude_entity_type_ids=[EntityTypeId.CAPVCD_1_0_0.value]  # noqa: E501
+                    )
                     # Get the list of cluster defined entities
                     entities: List[common_models.GenericClusterEntity] = clusters_page_results[PaginationKey.VALUES]  # noqa: E501
                     clusters = []
@@ -329,6 +337,22 @@ class DECluster:
             return self._nativeCluster.delete_cluster_by_id(cluster_id)
         return self._tkgCluster.delete_cluster_by_id(cluster_id, org=org)
 
+    def force_delete_cluster_by_id(self, cluster_id, org=None):
+        """Delete cluster using cluster id.
+
+        :param str cluster_id: id of the cluster to be deleted
+        :param str org:
+
+        :return: deleted cluster information
+        :rtype: str
+
+        :raises: ClusterNotFoundError
+        """
+        entity_svc = def_entity_svc.DefEntityService(self._cloudapi_client)
+        if entity_svc.is_native_entity(cluster_id):
+            return self._nativeCluster.force_delete_cluster_by_id(cluster_id)
+        return self._tkgCluster.delete_cluster_by_id(cluster_id, org=org)
+
     def get_upgrade_plan(self, cluster_name, org=None, vdc=None):
         """Get the upgrade plan for the given cluster name.
 
@@ -422,3 +446,24 @@ class DECluster:
         else:
             self._tkgCluster.unshare_cluster(cluster_id, cluster_name, users,
                                              org, vdc)
+
+    def force_delete_cluster(self, cluster_name, cluster_id=None, org=None, vdc=None):  # noqa: E501
+        """Force delete DEF cluster by name.
+
+        :param str cluster_name: name of the cluster
+        :param str cluster_id:
+        :param str org: name of the org
+        :param str vdc: name of the vdc
+
+        :return: deleted cluster info
+        :rtype: str
+        :raises ClusterNotFoundError, CseDuplicateClusterError
+        """
+        if cluster_id:
+            return self.force_delete_cluster_by_id(cluster_id)
+        cluster, entity_properties, is_native_cluster = \
+            self._get_tkg_s_and_native_clusters_by_name(cluster_name,
+                                                        org=org, vdc=vdc)
+        if is_native_cluster:
+            return self._nativeCluster.force_delete_cluster_by_id(cluster.id)
+        return self._tkgCluster.delete_cluster_by_id(cluster_id=entity_properties.get('id'))  # noqa: E501
