@@ -5,9 +5,10 @@
 """Utility methods used only by the CLI client."""
 
 import click
-from pyvcloud.vcd.client import Client
+from pyvcloud.vcd.client import Client, VcdApiVersionObj
 import pyvcloud.vcd.org as vcd_org
 import pyvcloud.vcd.utils as vcd_utils
+from pyvcloud.vcd.vcd_api_version import VCDApiVersion
 import requests
 import semantic_version
 import six
@@ -84,7 +85,9 @@ def cse_restore_session(ctx) -> None:
             log_headers=profiles.get('log_header'),
             log_bodies=profiles.get('log_body'))
         client.rehydrate_from_token(
-            profiles.get('token'), profiles.get('is_jwt_token'))
+            token=profiles.get('token'),
+            is_jwt_token=profiles.get('is_jwt_token')
+        )
 
         ctx.obj = {'client': client}
     else:
@@ -138,12 +141,12 @@ def _override_client(ctx) -> None:
                 # ToDo: Instead of float use proper version comparison
                 if is_cse_server_running_in_legacy_mode:
                     common_supported_api_versions = \
-                        [float(x) for x in common_supported_api_versions
-                            if float(x) < 35.0]
+                        [VCDApiVersion(x) for x in common_supported_api_versions  # noqa: E501
+                            if VCDApiVersion(x) < VcdApiVersionObj.VERSION_35.value]  # noqa: E501
                 else:
                     common_supported_api_versions = \
-                        [float(x) for x in common_supported_api_versions
-                            if float(x) >= 35.0]
+                        [VCDApiVersion(x) for x in common_supported_api_versions  # noqa: E501
+                            if VCDApiVersion(x) >= VcdApiVersionObj.VERSION_35.value]  # noqa: E501
 
                 cse_server_api_version = \
                     str(max(common_supported_api_versions))
@@ -180,7 +183,10 @@ def _override_client(ctx) -> None:
         log_requests=profiles.get('log_request'),
         log_headers=profiles.get('log_header'),
         log_bodies=profiles.get('log_body'))
-    client.rehydrate_from_token(profiles.get('token'), profiles.get('is_jwt_token'))  # noqa: E501
+    client.rehydrate_from_token(
+        token=profiles.get('token'),
+        is_jwt_token=profiles.get('is_jwt_token')
+    )
     ctx.obj['client'] = client
     ctx.obj['profiles'] = profiles
 
