@@ -8,6 +8,7 @@ from dataclasses import dataclass
 from enum import Enum
 from enum import unique
 
+from pyvcloud.vcd.client import VcdApiVersionObj
 import requests
 import semantic_version
 
@@ -65,7 +66,7 @@ MQTT_EXTENSION_VENDOR = 'VMWare'
 MQTT_EXTENSION_URN = 'urn:vcloud:extension-api:' + MQTT_EXTENSION_VENDOR + ':'\
                      + CSE_SERVICE_NAME + ':' + MQTT_EXTENSION_VERSION
 MQTT_EXTENSION_PRIORITY = 100
-MQTT_MIN_API_VERSION = 35.0
+MQTT_MIN_API_VERSION = VcdApiVersionObj.VERSION_35.value
 MQTT_TOKEN_NAME = "mqttCseToken"
 TOKEN_PATH = 'tokens'
 
@@ -89,6 +90,10 @@ VCENTER_LOGIN_ERROR_MSG = "vCenter login failed (check config file for "\
 # Request Id format for logging
 REQUEST_ID_FORMAT = 'Request Id: %(requestId)s | '
 
+# TKGM init values
+TKGM_DEFAULT_POD_NETWORK_CIDR = '100.96.0.0/11'
+TKGM_DEFAULT_SERVICE_CIDR = '100.64.0.0/13'
+
 
 @unique
 class OperationType(str, Enum):
@@ -138,12 +143,16 @@ class FlattenedClusterSpecKey2X(Enum):
     WORKERS_COUNT = 'topology.workers.count'
     WORKERS_SIZING_CLASS = 'topology.workers.sizingClass'
     WORKERS_STORAGE_PROFILE = 'topology.workers.storageProfile'
+    WORKERS_CPU_COUNT = 'topology.workers.cpu'
+    WORKERS_MEMORY_MB = 'topology.workers.memory'
     NFS_COUNT = 'topology.nfs.count'
     NFS_SIZING_CLASS = 'topology.nfs.sizingClass'
     NFS_STORAGE_PROFILE = 'topology.nfs.storageProfile'
     TEMPLATE_NAME = 'distribution.templateName'
     TEMPLATE_REVISION = 'distribution.templateRevision'
     EXPOSE = 'settings.network.expose'
+    POD_CIDR = 'settings.network.pods.cidrBlocks'
+    SVC_CIDR = 'settings.network.services.cidrBlocks'
 
 
 VALID_UPDATE_FIELDS_2X = \
@@ -202,10 +211,14 @@ class ClusterScriptFile(str, Enum):
     CONTROL_PLANE = 'control_plane.sh'
     NODE = 'node.sh'
 
+    CLOUDINIT_CONTROL_PLANE = 'cloud_init_control_plane.yaml'
+    CLOUDINIT_NODE = 'cloud_init_node.yaml'
+
     # Note that we need _ in the version instead of dots to allow
     # python package parsing methods to work well
     VERSION_1_X = "v1_x"
     VERSION_2_X = "v2_x"
+    VERSION_2_X_TKGM = "v2_x_tkgm"
 
 
 @unique
@@ -296,6 +309,42 @@ class LocalTemplateKey(str, Enum):
     MAX_CSE_VERSION = 'max_cse_version'
 
 
+@unique
+class TKGmTemplateKey(str, Enum):
+    """Enumerate the keys that define an imported TKGm template."""
+
+    CNI = 'cni'
+    CNI_VERSION = 'cni_version'
+    CONTAINER_RUNTIME = 'container_runtime'
+    CONTAINER_RUNTIME_VERSION = 'container_runtime_version'
+    CSE_VERSION = 'cse_version'
+    KIND = 'kind'
+    KUBERNETES = 'kubernetes'
+    KUBERNETES_VERSION = 'kubernetes_version'
+    NAME = 'name'
+    OS = 'os'
+    OS_VERSION = 'os_version'
+    REVISION = 'revision'
+
+
+@unique
+class TKGmProperty(str, Enum):
+    """Enumerate the keys in TKGm template ProductSection."""
+
+    BUILD_TIMESTAMP = 'BUILD_TIMESTAMP'
+    BUILD_DATE = 'BUILD_DATE'
+    CUSTOM_ROLE = 'CUSTOM_ROLE'
+    IMAGE_BUILDER_VERSION = 'IMAGE_BUILDER_VERSION'
+    DISTRO_NAME = 'DISTRO_NAME'
+    DISTRO_VERSION = 'DISTRO_VERSION'
+    DISTRO_ARCH = 'DISTRO_ARCH'
+    CNI_VERSION = 'CNI_VERSION'
+    CONTAINDERD_VERSION = 'CONTAINERD_VERSION'
+    KUBERNETES_SEMVER = 'KUBERNETES_SEMVER'
+    KUBERNETES_SOURCE_TYPE = 'KUBERNETES_SOURCE_TYPE'
+    VERSION = 'VERSION'
+
+
 @unique
 class RemoteTemplateKeyV1(str, Enum):
     """Enumerate the keys that define a template."""
@@ -404,6 +453,7 @@ class CseOperation(Enum):
     V36_CLUSTER_CONFIG = ('get config of DEF V36 cluster', '/cse/3.0/cluster/%s/config')  # noqa: E501
     V36_CLUSTER_UPDATE = ('update DEF V36 cluster', '/cse/3.0/cluster/%s', requests.codes.accepted)  # noqa: E501
     V36_CLUSTER_DELETE = ('delete DEF V36 cluster', '/cse/3.0/cluster/%s', requests.codes.accepted)  # noqa: E501
+    V36_CLUSTER_FORCE_DELETE = ('force delete DEF cluster', '/cse/3.0/cluster/%s/action/force-delete', requests.codes.accepted)  # noqa: E501
     V36_CLUSTER_UPGRADE_PLAN = ('get supported V36 DEF cluster upgrade paths', '/cse/3.0/cluster/%s/upgrade-plan')  # noqa: E501
     V36_NODE_DELETE = ('delete V36 DEF node', '/cse/3.0/cluster/%s/nfs/%s', requests.codes.accepted)  # noqa: E501
     V36_CLUSTER_ACL_LIST = ('list V36 cluster acl', '/cse/3.0/cluster/%s/acl')
@@ -418,6 +468,7 @@ class CseOperation(Enum):
     SYSTEM_INFO = ('get info of system', '/cse/system')
     SYSTEM_UPDATE = ('update system status', '/cse/system')
     TEMPLATE_LIST = ('list all templates', '/cse/templates')
+    TKGM_TEMPLATE_LIST = ('list TKGm templates', '/cse/templates/tkgm')
 
     V35_OVDC_LIST = ('list ovdcs for v35', '/cse/3.0/ovdcs')
     V35_ORG_VDC_LIST = ('list org VDCs', '/cse/3.0/orgvdcs')
@@ -633,6 +684,23 @@ CSE_SERVICE_ROLE_RIGHTS = [
     "vmware:tkgcluster: View"
 ]
 
+CSE_NATIVE_CLUSTER_ADMINISTRATOR_FULL_ACCESS_RIGHTS = {
+    'cse:nativeCluster: Administrator Full access'
+}
+
+CSE_NATIVE_CLUSTER_FULL_ACCESS_RIGHTS = {
+    'cse:nativeCluster: Full Access'
+}
+
+VAPP_DELETE_RIGHTS = {
+    'vApp: Delete'
+}
+
+DNAT_DELETE_RIGHTS = {
+    'Organization vDC Gateway: Configure NAT',
+    'Organization vDC Gateway: View'
+}
+
 
 @dataclass
 class DefEntityPhase:
@@ -712,7 +780,13 @@ USED_IP_ADDRESS_PAGE_SIZE = 10
 # Context headers
 TENANT_CONTEXT_HEADER = 'X-VMWARE-VCLOUD-TENANT-CONTEXT'
 AUTH_CONTEXT_HEADER = 'X-VMWARE-VCLOUD-AUTH-CONTEXT'
-VCLOUD_AUTHORIZATION_HEADER = 'X-vCloud-Authorization'
+
+
+# tkgm proxy key in CSE server config -> actual key as in http-proxy.config
+class TKGmProxyKey(Enum):
+    tkgm_http_proxy = 'http_proxy'
+    tkgm_https_proxy = 'https_proxy'
+    tkgm_no_proxy = 'no_proxy'
 
 
 @unique
@@ -722,8 +796,32 @@ class PostCustomizationStatus(Enum):
     SUCCESSFUL = 'successful'
 
 
-POST_CUSTOMIZATION_SCRIPT_EXECUTION_STATUS = 'post_customization_script_execution_status'  # noqa: E501
-POST_CUSTOMIZATION_SCRIPT_EXECUTION_FAILURE_REASON = 'post_customization_script_execution_failure_reason'  # noqa: E501
+@unique
+class PostCustomizationPhase(Enum):
+    NETWORK_CONFIGURATION = 'guestinfo.postcustomization.networkconfiguration.status'  # noqa: E501
+    STORE_SSH_KEY = 'guestinfo.postcustomization.store.sshkey.status'
+    KUBEADM_INIT = 'guestinfo.postcustomization.kubeinit.status'
+    KUBECTL_APPLY_CNI = 'guestinfo.postcustomization.kubectl.cni.install.status'  # noqa: E501
+    KUBECTL_APPLY_CPI = 'guestinfo.postcustomization.kubectl.cpi.install.status'  # noqa: E501
+    KUBECTL_APPLY_CSI = 'guestinfo.postcustomization.kubectl.csi.install.status'  # noqa: E501
+    KUBEADM_TOKEN_GENERATE = 'guestinfo.postcustomization.kubeadm.token.generate.status'  # noqa: E501
+    KUBEADM_NODE_JOIN = 'guestinfo.postcustomization.kubeadm.node.join.status'
+    PROXY_SETTING = 'guestinfo.postcustomization.proxy.setting.status'
+
+
+KUBEADM_TOKEN_INFO = 'guestinfo.postcustomization.kubeadm.token.info'
+KUBE_CONFIG = 'guestinfo.kubeconfig'
+POST_CUSTOMIZATION_SCRIPT_EXECUTION_STATUS = 'guestinfo.post_customization_script_execution_status'  # noqa: E501
+POST_CUSTOMIZATION_SCRIPT_EXECUTION_FAILURE_REASON = 'guestinfo.post_customization_script_execution_failure_reason'  # noqa: E501
 DEFAULT_POST_CUSTOMIZATION_STATUS_LIST = [cust_status.value for cust_status in PostCustomizationStatus]  # noqa: E501
 DEFAULT_POST_CUSTOMIZATION_POLL_SEC = 5
-DEFAULT_POST_CUSTOMIZATION_TIMEOUT_SEC = 180
+DEFAULT_POST_CUSTOMIZATION_TIMEOUT_SEC = 600
+DISK_ENABLE_UUID = 'disk.enableUUID'
+CLOUDINIT_GUEST_USERDATA = 'guestinfo.userdata'
+CLOUDINIT_GUEST_USERDATA_ENCODING = 'guestinfo.userdata.encoding'
+
+MAX_RDE_UPDATE_ATTEMPTS = 10
+
+# keys of the Behavior task response that gets sent to VCD
+BEHAVIOR_TASK_RESPONSE_RESULT_MESSAGE_KEY = "result"
+BEHAVIOR_TASK_RESPONSE_RESULT_CONTENT_MESSAGE_KEY = "resultContent"
