@@ -4,14 +4,15 @@
 
 """Utility module to perform operations which involve pyvcloud calls."""
 from datetime import datetime, timedelta
-import pathlib
 import time
 from typing import Optional
 import urllib
 
 import pyvcloud.vcd.client as vcd_client
 from pyvcloud.vcd.exceptions import EntityNotFoundException
+from pyvcloud.vcd.exceptions import OperationNotSupportedException
 import pyvcloud.vcd.org as vcd_org
+import pyvcloud.vcd.role as vcd_role
 from pyvcloud.vcd.utils import extract_id
 from pyvcloud.vcd.utils import get_admin_href
 from pyvcloud.vcd.utils import to_dict
@@ -20,7 +21,7 @@ from pyvcloud.vcd.vdc import VDC
 from pyvcloud.vcd.vm import VM
 import requests
 
-import container_service_extension.common.constants.server_constants as server_constants # noqa: E501
+import container_service_extension.common.constants.server_constants as server_constants  # noqa: E501
 import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
 from container_service_extension.common.utils.core_utils import extract_id_from_href  # noqa: E501
 from container_service_extension.common.utils.core_utils import NullPrinter
@@ -48,7 +49,6 @@ def raise_error_if_user_not_from_system_org(client: vcd_client.Client):
 
 def connect_vcd_user_via_token(
         tenant_auth_token: str,
-        is_jwt_token: bool,
         api_version: Optional[str]):
     server_config = get_server_runtime_config()
     if not api_version:
@@ -69,7 +69,10 @@ def connect_vcd_user_via_token(
         log_requests=log_wire,
         log_headers=log_wire,
         log_bodies=log_wire)
-    client_tenant.rehydrate_from_token(tenant_auth_token, is_jwt_token)
+    client_tenant.rehydrate_from_token(
+        token=tenant_auth_token,
+        is_jwt_token=True
+    )
     return client_tenant
 
 
@@ -257,14 +260,23 @@ def get_pvdc_id_from_pvdc_name(name, vc_name_in_vcd):
             client.logout()
 
 
-def upload_ova_to_catalog(client, catalog_name, filepath, update=False,
-                          org=None, org_name=None, logger=NULL_LOGGER,
-                          msg_update_callback=NullPrinter()):
+def upload_ova_to_catalog(
+        client,
+        source_filepath,
+        catalog_name,
+        catalog_item_name,
+        update=False,
+        org=None,
+        org_name=None,
+        logger=NULL_LOGGER,
+        msg_update_callback=NullPrinter()
+):
     """Upload local ova file to vCD catalog.
 
     :param pyvcloud.vcd.client.Client client:
-    :param str filepath: file path to the .ova file.
+    :param str source_filepath: file path to the .ova file.
     :param str catalog_name: name of catalog.
+    :param str catalog_item_name: name of the catalog item
     :param bool update: signals whether to overwrite an existing catalog
         item with this new one.
     :param pyvcloud.vcd.org.Org org: specific org to use.
@@ -280,7 +292,6 @@ def upload_ova_to_catalog(client, catalog_name, filepath, update=False,
     """
     if org is None:
         org = get_org(client, org_name=org_name)
-    catalog_item_name = pathlib.Path(filepath).name
     if update:
         try:
             msg = f"Update flag set. Checking catalog '{catalog_name}' for " \
@@ -320,7 +331,11 @@ def upload_ova_to_catalog(client, catalog_name, filepath, update=False,
     msg_update_callback.info(msg)
     logger.info(msg)
 
-    org.upload_ovf(catalog_name, filepath)
+    org.upload_ovf(
+        catalog_name=catalog_name,
+        file_name=source_filepath,
+        item_name=catalog_item_name
+    )
     org.reload()
     wait_for_catalog_item_to_resolve(client, catalog_name, catalog_item_name,
                                      org=org)
@@ -511,23 +526,21 @@ def get_storage_profile_name_of_first_vm_in_vapp(vapp):
     return storage_profile_name
 
 
-def get_cloudapi_client_from_vcd_client(client: vcd_client.Client,
-                                        logger_debug=NULL_LOGGER,
-                                        logger_wire=NULL_LOGGER):
+def get_cloudapi_client_from_vcd_client(
+        client: vcd_client.Client,
+        logger_debug=NULL_LOGGER,
+        logger_wire=NULL_LOGGER
+):
     token = client.get_access_token()
-    is_jwt = True
-    if not token:
-        token = client.get_xvcloud_authorization_token()
-        is_jwt = False
     return cloud_api_client.CloudApiClient(
         base_url=client.get_cloudapi_uri(),
         token=token,
-        is_jwt_token=is_jwt,
         api_version=client.get_api_version(),
         logger_debug=logger_debug,
         logger_wire=logger_wire,
         verify_ssl=client._verify_ssl_certs,
-        is_sys_admin=client.is_sysadmin())
+        is_sys_admin=client.is_sysadmin()
+    )
 
 
 def get_all_ovdcs(client: vcd_client.Client):
@@ -693,7 +706,7 @@ def get_vm_extra_config_element(vm: VM, element_name: str) -> str:
     return vm_extra_config_elements.get(element_name)
 
 
-def wait_for_completion_of_post_customization_step(
+def wait_for_completion_of_post_customization_procedure(
         vm: VM,
         customization_phase: str,
         timeout=server_constants.DEFAULT_POST_CUSTOMIZATION_TIMEOUT_SEC,
@@ -733,6 +746,7 @@ def wait_for_completion_of_post_customization_step(
     remaining_statuses = list(expected_target_status_list)
 
     while True:
+        vm.reload()
         new_status = get_vm_extra_config_element(vm, customization_phase)
         if new_status not in remaining_statuses:
             logger.error(f"Invalid VM Post guest customization status:{new_status}")  # noqa: E501
@@ -740,8 +754,8 @@ def wait_for_completion_of_post_customization_step(
         # update the remaining statuses on status change
         if new_status != current_status:
             remaining_statuses.remove(current_status)
-            logger.info(f"Post guest customization phase {customization_phase } in {new_status}")  # noqa: E501
             current_status = new_status
+        logger.info(f"Post guest customization phase {customization_phase} is {new_status}")  # noqa: E501
         # Check for successful customization: reaching last status between
         if new_status == expected_target_status_list[-1]:
             return new_status
@@ -751,10 +765,59 @@ def wait_for_completion_of_post_customization_step(
         if script_execution_status and int(script_execution_status) != 0:
             script_execution_failure_reason = get_vm_extra_config_element(vm, server_constants.POST_CUSTOMIZATION_SCRIPT_EXECUTION_FAILURE_REASON)  # noqa: E501
             logger.error(f"VM Post guest customization script failed with error:{script_execution_failure_reason}")  # noqa: E501
-            raise exceptions.ScriptExecutionError
+            raise exceptions.ScriptExecutionError(error_message=script_execution_failure_reason)  # noqa: E501
 
         if datetime.now() - start_time > timedelta(seconds=timeout):
             break
         time.sleep(poll_frequency)
-    logger.error("VM Post guest customization failed due to timeout")  # noqa: E501
+    logger.error(f"VM Post guest customization failed due to timeout({timeout} sec)")  # noqa: E501
     raise exceptions.PostCustomizationTimeoutError
+
+
+def get_missing_rights_for_cluster_force_delete(
+        client: vcd_client.Client,
+        is_cluster_owner=False,
+        logger=NULL_LOGGER) -> set:
+    """
+    Find the missing rights for force-deleting the cluster.
+
+    :param vcd_client.Client client: current client
+    :param bool is_cluster_owner: true if the client is cluster owner
+    :param logging.Logger logger: logger to record errors.
+    :return: all missing rights
+    :rtype: set
+    :raises: OperationNotSupportedException
+    :raises: Exception for any unexpected errors
+    """
+    missing_rights = set()
+    role_name = ''
+    try:
+        role_name = get_user_role_name(client)
+        org_obj = vcd_org.Org(client, resource=client.get_org())
+        role_obj = vcd_role.Role(client, resource=org_obj.get_role_resource(role_name))  # noqa: E501
+        logger.debug(f"role_name:{role_name}")
+        role_rights = set()
+
+        for right_dict in role_obj.list_rights():
+            role_rights.update(right_dict.values())
+
+        logger.debug(f"rights of role:{role_name}--{role_rights}")
+        missing_rights = set()
+        if not server_constants.VAPP_DELETE_RIGHTS.issubset(role_rights):
+            missing_rights.update(server_constants.VAPP_DELETE_RIGHTS)
+        if not server_constants.DNAT_DELETE_RIGHTS.issubset(role_rights):
+            missing_rights.update(server_constants.DNAT_DELETE_RIGHTS)
+        if is_cluster_owner:
+            if not server_constants.CSE_NATIVE_CLUSTER_FULL_ACCESS_RIGHTS.issubset(role_rights) and not server_constants.CSE_NATIVE_CLUSTER_ADMINISTRATOR_FULL_ACCESS_RIGHTS.issubset(role_rights):  # noqa: E501
+                missing_rights.update(server_constants.CSE_NATIVE_CLUSTER_FULL_ACCESS_RIGHTS)  # noqa: E501
+        else:
+            if not server_constants.CSE_NATIVE_CLUSTER_ADMINISTRATOR_FULL_ACCESS_RIGHTS.issubset(role_rights):  # noqa: E501
+                missing_rights.update(server_constants.CSE_NATIVE_CLUSTER_ADMINISTRATOR_FULL_ACCESS_RIGHTS)  # noqa: E501
+    except OperationNotSupportedException as err:
+        logger.warning(f"Cannot determine the rights associated with role:{role_name} {err}")  # noqa: E501
+        raise
+    except Exception as err:
+        logger.warning(f"Cannot determine the rights associated with role:{role_name} {err}")  # noqa: E501
+        raise
+    logger.debug(f"missing rights of role:{role_name}--{missing_rights}")
+    return missing_rights
