@@ -4,10 +4,11 @@
 
 import json
 
-import pyvcloud.vcd.utils as pyvcd_utils
+from pyvcloud.vcd.utils import extract_id
 
 from container_service_extension.common.constants.server_constants import CLUSTER_ENTITY  # noqa: E501
 from container_service_extension.common.constants.server_constants import LocalTemplateKey  # noqa: E501
+from container_service_extension.common.constants.server_constants import TKGmTemplateKey  # noqa: E501
 import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
 from container_service_extension.common.constants.shared_constants import AccessControlKey  # noqa: E501
 from container_service_extension.common.constants.shared_constants import ClusterAclKey  # noqa: E501
@@ -90,10 +91,7 @@ def get_payload_for_install_server(params):
     return {
         PayloadKey.TYPE: CseOperation.SERVICE_INSTALL.telemetry_table,
         PayloadKey.WAS_DECRYPTION_SKIPPED: bool(params.get(PayloadKey.WAS_DECRYPTION_SKIPPED)),  # noqa: E501
-        PayloadKey.WAS_PKS_CONFIG_FILE_PROVIDED: bool(params.get(PayloadKey.WAS_PKS_CONFIG_FILE_PROVIDED)),  # noqa: E501
-        PayloadKey.WERE_TEMPLATES_SKIPPED: bool(params.get(PayloadKey.WERE_TEMPLATES_SKIPPED)),  # noqa: E501
-        PayloadKey.WAS_TEMP_VAPP_RETAINED: bool(params.get(PayloadKey.WAS_TEMP_VAPP_RETAINED)),  # noqa: E501
-        PayloadKey.WAS_SSH_KEY_SPECIFIED: bool(params.get(PayloadKey.WAS_SSH_KEY_SPECIFIED))  # noqa: E501
+        PayloadKey.WAS_PKS_CONFIG_FILE_PROVIDED: bool(params.get(PayloadKey.WAS_PKS_CONFIG_FILE_PROVIDED))  # noqa: E501
     }
 
 
@@ -109,12 +107,7 @@ def get_payload_for_upgrade_server(params):
     return {
         PayloadKey.TYPE: CseOperation.SERVICE_UPGRADE.telemetry_table,
         PayloadKey.SOURCE_CSE_VERSION: params.get(PayloadKey.SOURCE_CSE_VERSION),  # noqa: E501
-        PayloadKey.SOURCE_VCD_API_VERSION: params.get(PayloadKey.SOURCE_VCD_API_VERSION),  # noqa: E501
         PayloadKey.TARGET_CSE_VERSION: params.get(PayloadKey.TARGET_CSE_VERSION),  # noqa: E501
-        PayloadKey.TARGET_VCD_API_VERSION: params.get(PayloadKey.TARGET_VCD_API_VERSION),  # noqa: E501
-        PayloadKey.WERE_TEMPLATES_SKIPPED: bool(params.get(PayloadKey.WERE_TEMPLATES_SKIPPED)),  # noqa: E501
-        PayloadKey.WAS_TEMP_VAPP_RETAINED: bool(params.get(PayloadKey.WAS_TEMP_VAPP_RETAINED)),  # noqa: E501
-        PayloadKey.WAS_SSH_KEY_SPECIFIED: bool(params.get(PayloadKey.WAS_SSH_KEY_SPECIFIED))  # noqa: E501
     }
 
 
@@ -135,6 +128,34 @@ def get_payload_for_run_server(params):
     }
 
 
+def get_payload_for_import_template(params):
+    """Construct telemetry payload of template import operation.
+
+    :param params: parameters provided to the operation
+
+    :return: json telemetry data for the operation
+
+    :type: dict
+    """
+    return {
+        PayloadKey.TYPE: CseOperation.TEMPLATE_IMPORT.telemetry_table,
+        PayloadKey.CNI: params.get(TKGmTemplateKey.CNI),
+        PayloadKey.CNI_VERSION: params.get(TKGmTemplateKey.CNI_VERSION),
+        PayloadKey.CONTAINER_RUNTIME: params.get(TKGmTemplateKey.CONTAINER_RUNTIME),  # noqa: E501
+        PayloadKey.CONTAINER_RUNTIME_VERSION: params.get(TKGmTemplateKey.CONTAINER_RUNTIME_VERSION),  # noqa: E501
+        PayloadKey.CSE_VERSION: str(params.get(TKGmTemplateKey.CSE_VERSION)),
+        PayloadKey.KIND: params.get(TKGmTemplateKey.KIND),
+        PayloadKey.K8S_DISTRIBUTION: params.get(TKGmTemplateKey.KUBERNETES),
+        PayloadKey.K8S_VERSION: params.get(TKGmTemplateKey.KUBERNETES_VERSION),
+        PayloadKey.TEMPLATE_NAME: params.get(TKGmTemplateKey.NAME),
+        PayloadKey.OS: params.get(TKGmTemplateKey.OS),
+        PayloadKey.OS_VERSION: params.get(TKGmTemplateKey.OS_VERSION),
+        PayloadKey.TEMPLATE_REVISION: params.get(TKGmTemplateKey.REVISION),
+        PayloadKey.WAS_DECRYPTION_SKIPPED: bool(params.get(PayloadKey.WAS_DECRYPTION_SKIPPED)),  # noqa: E501
+        PayloadKey.WERE_TEMPLATES_FORCE_UPDATED: bool(params.get(PayloadKey.WERE_TEMPLATES_FORCE_UPDATED)),  # noqa: E501
+    }
+
+
 def get_payload_for_install_template(params):
     """Construct telemetry payload of template install operation.
 
@@ -561,7 +582,7 @@ def get_payload_for_v35_cluster_info(params):
     """
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_INFO.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
     }
@@ -579,7 +600,7 @@ def get_payload_for_v35_cluster_config(params):
     def_entity = params.get(CLUSTER_ENTITY)
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_CONFIG.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: def_entity.entity.kind,
         PayloadKey.TEMPLATE_NAME: def_entity.entity.spec.k8_distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: def_entity.entity.spec.k8_distribution.template_revision,  # noqa: E501
@@ -600,7 +621,7 @@ def get_payload_for_v35_cluster_apply(params):
     def_entity = params.get(CLUSTER_ENTITY)
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_APPLY.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: def_entity.entity.kind,
         PayloadKey.TEMPLATE_NAME: def_entity.entity.spec.k8_distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: def_entity.entity.spec.k8_distribution.template_revision,  # noqa: E501
@@ -626,7 +647,7 @@ def get_payload_for_v35_cluster_delete(params):
     def_entity = params.get(CLUSTER_ENTITY)
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_DELETE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: def_entity.entity.kind,
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
@@ -645,7 +666,7 @@ def get_payload_for_v35_cluster_upgrade_plan(params):
     def_entity = params.get(CLUSTER_ENTITY)
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_UPGRADE_PLAN.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: def_entity.entity.kind,
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
@@ -664,7 +685,7 @@ def get_payload_for_v35_cluster_upgrade(params):
     def_entity = params.get(CLUSTER_ENTITY)
     return {
         PayloadKey.TYPE: CseOperation.V35_CLUSTER_UPGRADE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: def_entity.entity.kind,
         PayloadKey.TEMPLATE_NAME: def_entity.entity.spec.k8_distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: def_entity.entity.spec.k8_distribution.template_revision,  # noqa: E501
@@ -684,7 +705,7 @@ def get_payload_for_v35_node_delete(params):
     """
     return {
         PayloadKey.TYPE: CseOperation.V35_NODE_DELETE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
         PayloadKey.NODE_NAME: params.get(PayloadKey.NODE_NAME),
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
@@ -719,7 +740,7 @@ def get_payload_for_v36_cluster_info(params):
     """
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_INFO.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
     }
@@ -738,7 +759,7 @@ def get_payload_for_v36_cluster_config(params):
     native_entity: rde_2_x.NativeEntity = def_entity.entity
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_CONFIG.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),  # noqa: E501
         PayloadKey.CLUSTER_KIND: native_entity.kind,
         PayloadKey.TEMPLATE_NAME: native_entity.spec.distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: native_entity.spec.distribution.template_revision,  # noqa: E501
@@ -760,7 +781,7 @@ def get_payload_for_v36_cluster_apply(params):
     native_entity: rde_2_x.NativeEntity = def_entity.entity
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_APPLY.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: native_entity.kind,
         PayloadKey.TEMPLATE_NAME: native_entity.spec.distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: native_entity.spec.distribution.template_revision,  # noqa: E501
@@ -787,7 +808,7 @@ def get_payload_for_v36_cluster_delete(params):
     native_entity: rde_2_x.NativeEntity = def_entity.entity
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_DELETE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: native_entity.kind,
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
@@ -807,7 +828,7 @@ def get_payload_for_v36_cluster_upgrade_plan(params):
     native_entity: rde_2_x.NativeEntity = def_entity.entity
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_UPGRADE_PLAN.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: native_entity.kind,
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
@@ -827,7 +848,7 @@ def get_payload_for_v36_cluster_upgrade(params):
     native_entity: rde_2_x.NativeEntity = def_entity.entity
     return {
         PayloadKey.TYPE: CseOperation.V36_CLUSTER_UPGRADE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(def_entity.id)),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(def_entity.id)),
         PayloadKey.CLUSTER_KIND: native_entity.kind,
         PayloadKey.TEMPLATE_NAME: native_entity.spec.distribution.template_name,  # noqa: E501
         PayloadKey.TEMPLATE_REVISION: native_entity.spec.distribution.template_revision,  # noqa: E501
@@ -847,7 +868,7 @@ def get_payload_for_v36_node_delete(params):
     """
     return {
         PayloadKey.TYPE: CseOperation.V36_NODE_DELETE.telemetry_table,
-        PayloadKey.CLUSTER_ID: uuid_hash(pyvcd_utils.extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
+        PayloadKey.CLUSTER_ID: uuid_hash(extract_id(params.get(PayloadKey.CLUSTER_ID))),  # noqa: E501
         PayloadKey.NODE_NAME: params.get(PayloadKey.NODE_NAME),
         PayloadKey.SOURCE_ID: SourceMap.get_source_id(params.get(PayloadKey.SOURCE_DESCRIPTION)),  # noqa: E501
         PayloadKey.SOURCE_DESCRIPTION: params.get(PayloadKey.SOURCE_DESCRIPTION)  # noqa: E501
