@@ -6,9 +6,12 @@ import functools
 import json
 from typing import List, Tuple, Union
 
-from pyvcloud.vcd.client import ApiVersion
+import pyvcloud.vcd.client as vcd_client
+from pyvcloud.vcd.vcd_api_version import VCDApiVersion
+from requests import codes
 from requests.exceptions import HTTPError
 
+import container_service_extension.common.constants.server_constants as server_constants  # noqa: E501
 import container_service_extension.common.constants.shared_constants as shared_constants  # noqa: E501
 from container_service_extension.common.constants.shared_constants import CSE_PAGINATION_DEFAULT_PAGE_SIZE, PaginationKey  # noqa: E501
 from container_service_extension.common.constants.shared_constants import CSE_PAGINATION_FIRST_PAGE_NUMBER  # noqa: E501
@@ -50,12 +53,12 @@ def handle_entity_service_exception(func):
         except HTTPError as error:
             response_dict = json.loads(error.response.text)
             error_message = response_dict.get('message')
-            LOGGER.error(error_message)
+            LOGGER.error(error_message, exc_info=True)
             raise cse_exception.DefEntityServiceError(
                 error_message=error_message,
                 minor_error_code=MinorErrorCode.DEFAULT_ERROR_CODE)
         except Exception as error:
-            LOGGER.error(error)
+            LOGGER.error(str(error))
             raise error
         return result
     return exception_handler_wrapper
@@ -218,7 +221,8 @@ class DefEntityService:
     def get_all_entities_per_page_by_interface(self, vendor: str, nss: str, version: str,  # noqa: E501
                                                filters: dict = None,
                                                page_number: int = CSE_PAGINATION_FIRST_PAGE_NUMBER,  # noqa: E501
-                                               page_size: int = CSE_PAGINATION_DEFAULT_PAGE_SIZE):  # noqa: E501
+                                               page_size: int = CSE_PAGINATION_DEFAULT_PAGE_SIZE, # noqa: E501
+                                               exclude_entity_type_ids: list = None):  # noqa: E501
         """Get a page of entities belonging to an interface.
 
         An interface is uniquely identified by properties vendor, nss and
@@ -230,11 +234,23 @@ class DefEntityService:
         :param dict filters:
         :param int page_number:
         :param int page_size:
+        :param list exclude_entity_type_ids: exclude entity types
 
         :return: Generator of entities of that interface type
         :rtype: Generator[List, None, None]
         """
         filter_string = utils.construct_filter_string(filters)
+
+        exclude_entity_type_filter = ''
+        if exclude_entity_type_ids:
+            exclude_entity_type_filter = ";".join([f"entityType!={entity_type}" for entity_type in exclude_entity_type_ids])  # noqa: E501
+
+        if filter_string:
+            if exclude_entity_type_filter:
+                filter_string += f";{exclude_entity_type_filter}"
+        else:
+            filter_string = exclude_entity_type_filter
+
         query_string = f"page={page_number}&pageSize={page_size}"
         if filter_string:
             query_string = f"filter={filter_string}&{query_string}"
@@ -255,10 +271,30 @@ class DefEntityService:
         result[PaginationKey.VALUES] = entity_list
         return result
 
+    def _form_updated_entity(self, entity_id: str, changes: dict):
+        entity = self.get_entity(entity_id)
+        # get entity call updates _cloudapi_client headers to get etag
+        etag = self._cloudapi_client.get_last_response_etag()
+
+        # form updated RDE
+        for attr_str, value in changes.items():
+            curr_obj = entity
+            attrs: list = attr_str.split('.')
+            last_attr: str = attrs.pop()
+            for attr in attrs:
+                try:
+                    curr_obj = getattr(curr_obj, attr)
+                except AttributeError as err:
+                    LOGGER.error(f"missing attribute [{attr}] in [{attr_str}] when updating RDE: {err}")   # noqa: E501
+                    raise
+            setattr(curr_obj, last_attr, value)
+        return entity, etag
+
     @handle_entity_service_exception
-    def update_entity(self, entity_id: str, entity: DefEntity,
+    def update_entity(self, entity_id: str, entity: DefEntity = None,
                       invoke_hooks=False,
-                      is_request_async=False) -> Union[DefEntity, Tuple[DefEntity, dict]]:  # noqa: E501
+                      is_request_async=False,
+                      changes: dict = None) -> Union[DefEntity, Tuple[DefEntity, dict]]:  # noqa: E501
         """Update entity instance.
 
         :param str entity_id: Id of the entity to be updated.
@@ -268,50 +304,85 @@ class DefEntityService:
         :param bool is_request_async: The request is intended to be
             asynchronous if this flag is set, href of the task is returned
             in addition to the response body
+        :param dict changes: dictionary of changes for the rde. The key
+            indicates the field updated, e.g. 'entity.status'. The value
+            indicates the value for this field.
 
         :return: Updated entity or Updated entity and response headers
         :rtype: Union[DefEntity, Tuple[DefEntity, dict]]
+
+        The 'changes' parameter takes precedence over 'entity', and at least
+        one of these fields is required. Also, 'changes' is required when
+        api version >= 36.0; this allows getting the entity so that the current
+        entity is not overwritten.
+        etag usage is only supported for api version >= 36.0.
         """
+        if entity is None and changes is None:
+            raise Exception("at least 'entity' or 'changes' must not be None")
+        vcd_api_version = self._cloudapi_client.get_vcd_api_version()
+        api_at_least_36: bool = vcd_api_version >= VCDApiVersion(vcd_client.ApiVersion.VERSION_36.value)  # noqa: E501
+        if api_at_least_36 and changes is None:
+            raise Exception("changes must be specified when client has api "
+                            "version >= 36.0")
         resource_url_relative_path = f"{CloudApiResource.ENTITIES}/{entity_id}"
-        vcd_api_version = self._cloudapi_client.get_api_version()
         # TODO Float conversions must be changed to Semantic versioning.
         # TODO: Also include any persona having Administrator:FullControl
         #  on CSE:nativeCluster
-        if float(vcd_api_version) >= float(ApiVersion.VERSION_36.value) and \
-                self._cloudapi_client.is_sys_admin and not invoke_hooks:
+        if api_at_least_36 and self._cloudapi_client.is_sys_admin and not invoke_hooks:  # noqa: E501
             resource_url_relative_path += f"?invokeHooks={str(invoke_hooks).lower()}"  # noqa: E501
 
-        payload: dict = entity.to_dict()
+        for _ in range(server_constants.MAX_RDE_UPDATE_ATTEMPTS):
+            # get entity
+            etag = ""
+            if changes:
+                entity, etag = self._form_updated_entity(entity_id, changes)
 
-        # Prevent users with rights <= EDIT/VIEW on CSE:NATIVECLUSTER from
-        # updating "private" property of RDE "status" section
-        # TODO: Replace sys admin check with FULL CONTROL rights check on
-        #  CSE:NATIVECLUSTER. Users with no FULL CONTROL rights cannot update
-        #  private property of entity->status.
-        if not self._cloudapi_client.is_sys_admin:
-            payload.get('entity', {}).get('status', {}).pop('private', None)
+            payload: dict = entity.to_dict()
+
+            entity_kind = payload.get('entity', {}).get('kind', "")
+            # Prevent users with rights <= EDIT/VIEW on CSE:NATIVECLUSTER from
+            # updating "private" property of RDE "status" section
+            # TODO: Replace sys admin check with FULL CONTROL rights check on
+            #  CSE:NATIVECLUSTER. Users with no FULL CONTROL rights cannot
+            #  update private property of entity->status.
+            if not self._cloudapi_client.is_sys_admin:
+                # HACK: TKGm uses the principle of least privileges and
+                # does not use a ClusterAuthor role at all. Instead it only
+                # uses a ClusterAdmin role which always has FC rights.
+                # Hence it does not escalate to sys-admin and hence fails
+                # this check. Hence skip TKGm clusters.
+                # The correct fix for this is to implement the
+                # to-do mentioned above. That is covered by VCDA-2969.
+                if entity_kind != shared_constants.ClusterEntityKind.TKG_M.value:  # noqa: E501
+                    payload.get('entity', {}).get('status', {}).pop('private', None)  # noqa: E501
 
-        if is_request_async:
             # if request is async, return the task href in
             # x_vmware_vcloud_task_location header
             # TODO: Use the Http response status code to decide which
             #   header name to use for task_href
             #   202 - location header,
             #   200 - xvcloud-task-location needs to be used
-            response_body, headers = self._cloudapi_client.do_request(
-                method=RequestMethod.PUT,
-                cloudapi_version=CloudApiVersion.VERSION_1_0_0,
-                resource_url_relative_path=resource_url_relative_path,
-                payload=payload,
-                return_response_headers=is_request_async)
-            return DefEntity(**response_body), headers[HttpResponseHeader.X_VMWARE_VCLOUD_TASK_LOCATION.value]  # noqa: E501
-        else:
-            response_body = self._cloudapi_client.do_request(
-                method=RequestMethod.PUT,
-                cloudapi_version=CloudApiVersion.VERSION_1_0_0,
-                resource_url_relative_path=resource_url_relative_path,
-                payload=payload)
-            return DefEntity(**response_body)
+            try:
+                additional_request_headers = {"If-Match": etag} if api_at_least_36 else None  # noqa: E501
+                response_body, headers = self._cloudapi_client.do_request(
+                    method=RequestMethod.PUT,
+                    cloudapi_version=CloudApiVersion.VERSION_1_0_0,
+                    resource_url_relative_path=resource_url_relative_path,
+                    payload=payload,
+                    return_response_headers=True,
+                    additional_request_headers=additional_request_headers
+                )
+
+                def_entity = DefEntity(**response_body)
+                if is_request_async:
+                    return def_entity, headers[HttpResponseHeader.X_VMWARE_VCLOUD_TASK_LOCATION.value]  # noqa: E501
+                return def_entity
+            except Exception:
+                last_cloudapi_response = self._cloudapi_client.get_last_response()  # noqa: E501
+                if last_cloudapi_response.status_code == codes.precondition_failed:  # noqa: E501
+                    continue
+                raise
+        raise Exception(f"failed to update RDE after {server_constants.MAX_RDE_UPDATE_ATTEMPTS} attempts")  # noqa: E501
 
     @handle_entity_service_exception
     def get_entity(self, entity_id: str) -> DefEntity:
@@ -320,7 +391,7 @@ class DefEntityService:
         :param str entity_id: Id of the entity.
         :return: Details of the entity.
         :rtype: DefEntity
-        """
+    """
         response_body = self._cloudapi_client.do_request(
             method=RequestMethod.GET,
             cloudapi_version=CloudApiVersion.VERSION_1_0_0,
@@ -406,6 +477,34 @@ class DefEntityService:
                                               filters=filters):
             return entity
 
+    @handle_entity_service_exception
+    def list_all_native_rde_by_name_and_rde_version(self,
+                                                    name: str,
+                                                    version: str,
+                                                    filters: dict = None) -> DefEntity:  # noqa: E501
+        """List all native RDE given its name and RDE version.
+
+        This function has been introduced to make it easier to iterate over
+            all the RDE with the same name.
+
+        :param str name: Name of the native cluster.
+        :param str version: RDE version
+        :param dict filters: dictionary representing filters
+        :return: List of entities
+        :rtype: Generator[DefEntity, None, None]
+        """
+        if not filters:
+            filters = {}
+        filters[def_constants.RDEFilterKey.NAME.value] = name
+        native_entity_type: DefEntityType = \
+            def_utils.get_rde_metadata(version)[def_constants.RDEMetadataKey.ENTITY_TYPE]  # noqa: E501
+        for entity in \
+            self.list_entities_by_entity_type(vendor=native_entity_type.vendor,
+                                              nss=native_entity_type.nss,
+                                              version=native_entity_type.version,  # noqa: E501
+                                              filters=filters):
+            yield entity
+
     @handle_entity_service_exception
     def delete_entity(self, entity_id: str, invoke_hooks: bool = False, is_request_async=False) -> Union[dict, Tuple[dict, dict]]:  # noqa: E501
         """Delete the defined entity.
@@ -421,13 +520,15 @@ class DefEntityService:
         """
         # response will be a tuple (response_body, response_header) if
         # is_request_async is true. Else, it will be just response_body
-        vcd_api_version = self._cloudapi_client.get_api_version()
+        vcd_api_version = self._cloudapi_client.get_vcd_api_version()
         resource_url_relative_path = f"{CloudApiResource.ENTITIES}/{entity_id}"
 
         # TODO: Also include any persona having Administrator:FullControl
         #  on CSE:nativeCluster
-        if float(vcd_api_version) >= float(ApiVersion.VERSION_36.value) and \
-                self._cloudapi_client.is_sys_admin and not invoke_hooks:
+        if (
+            vcd_api_version >= VCDApiVersion(vcd_client.ApiVersion.VERSION_36.value)  # noqa: E501
+            and self._cloudapi_client.is_sys_admin and not invoke_hooks
+        ):
             resource_url_relative_path += f"?invokeHooks={str(invoke_hooks).lower()}"  # noqa: E501
 
         response = self._cloudapi_client.do_request(
@@ -444,6 +545,35 @@ class DefEntityService:
             return response[0], response[1][HttpResponseHeader.LOCATION.value]
         return response
 
+    @handle_entity_service_exception
+    def force_delete_entity(self, entity_id: str, is_request_async=False) -> Union[dict, Tuple[dict, dict]]:  # noqa: E501
+        """Delete the defined entity without invoking hooks.
+
+        :param str entity_id: Id of the entity.
+        :param bool is_request_async: The request is intended to be
+            asynchronous if this flag is set, href of the task is returned
+            in addition to the response body
+
+        :return: response body or response body and response headers
+        :rtype: Union[dict, Tuple[dict, dict]]
+        """
+        # response will be a tuple (response_body, response_header) if
+        # is_request_async is true. Else, it will be just response_body
+        vcd_api_version = self._cloudapi_client.get_vcd_api_version()
+        resource_url_relative_path = f"{CloudApiResource.ENTITIES}/{entity_id}"
+
+        if vcd_api_version >= VCDApiVersion(vcd_client.ApiVersion.VERSION_36.value):  # noqa: E501
+            resource_url_relative_path += "?invokeHooks=false"
+
+        response = self._cloudapi_client.do_request(
+            method=RequestMethod.DELETE,
+            cloudapi_version=CloudApiVersion.VERSION_1_0_0,
+            resource_url_relative_path=resource_url_relative_path,  # noqa: E501
+            return_response_headers=is_request_async)
+        if is_request_async:
+            return response[0], response[1][HttpResponseHeader.LOCATION.value]
+        return response
+
     @handle_entity_service_exception
     def resolve_entity(self, entity_id: str, entity_type_id: str = None) -> DefEntity:  # noqa: E501
         """Resolve the entity.
@@ -483,9 +613,12 @@ class DefEntityService:
         return def_constants.DEF_NATIVE_ENTITY_TYPE_NSS in response_body['entityType']  # noqa: E501
 
     @handle_entity_service_exception
-    def upgrade_entity(self, entity_id: str,
-                       upgraded_native_entity: AbstractNativeEntity,
-                       target_entity_type_id: str):
+    def upgrade_entity(
+            self,
+            entity_id: str,
+            upgraded_native_entity: AbstractNativeEntity,
+            target_entity_type_id: str
+    ):
         """Upgrade entity type of the entity to the specified one.
 
         :param str entity_id: ID of the entity to upgrade
@@ -496,10 +629,9 @@ class DefEntityService:
         :return: DefEntity representing the upgraded defined entity
         :rtype: DefEntity
         """
-        rde = self.get_entity(entity_id)
-        rde.entity = upgraded_native_entity
-
-        # Update only the entityType property
-        rde.entityType = target_entity_type_id
-
-        return self.update_entity(entity_id, rde, invoke_hooks=False)
+        changes = {
+            'entity': upgraded_native_entity,
+            'entityType': target_entity_type_id
+        }
+        return self.update_entity(entity_id, invoke_hooks=False,
+                                  changes=changes)
