@@ -79,6 +79,17 @@ TEMPLATE_HANDLERS = [
                 'handler': template_handler.template_list
             }
         }
+    },
+    {
+        'url': "cse/templates/tkgm",
+        RequestMethod.GET: {
+            ('36.0', ): {
+                'allowed_params': [],
+                'required_params': [],
+                'operation': CseOperation.TEMPLATE_LIST,
+                'handler': template_handler.tkgm_template_list
+            }
+        }
     }
 ]
 
@@ -583,6 +594,20 @@ CLUSTER_HANDLERS = [
             }
         },
     },
+    {
+        'url': f"cse/3.0/cluster/${RequestKey.CLUSTER_ID}/action/force-delete",
+        RequestMethod.POST: {
+            ('36.0',): {
+                'allowed_params': [],
+                'required_params': [],
+                'verify_payload': False,
+                'payload_type': '*',
+                'operation': CseOperation.V36_CLUSTER_FORCE_DELETE,
+                'handler': cluster_handler.cluster_force_delete,
+                'feature_flags': ['non_legacy_api']
+            }
+        },
+    },
     {
         'url': f"cse/3.0/cluster/${RequestKey.CLUSTER_ID}/nfs/${RequestKey.NODE_NAME}",  # noqa: E501
         RequestMethod.DELETE: {
@@ -976,18 +1001,21 @@ def process_request(message, mqtt_publisher=None):
     request_data['url'] = url
 
     # extract out the authorization token
-    tenant_auth_token = message['headers'].get('x-vcloud-authorization')
-    is_jwt_token = False
+    tenant_auth_token = ""
     auth_header = message['headers'].get('Authorization')
     if auth_header:
         tokens = auth_header.split(" ")
         if len(tokens) == 2 and tokens[0].lower() == 'bearer':
             tenant_auth_token = tokens[1]
-            is_jwt_token = True
+    if not tenant_auth_token:
+        LOGGER.debug("Invalid authorization token.")
 
     # create operation context
     operation_ctx = ctx.OperationContext(
-        tenant_auth_token, is_jwt=is_jwt_token, request_id=message['id'], mqtt_publisher=mqtt_publisher)  # noqa: E501
+        tenant_auth_token,
+        request_id=message['id'],
+        mqtt_publisher=mqtt_publisher
+    )
 
     try:
         body_content = handler_method(request_data, operation_ctx)
